// =====================================================
//
//  lazmla
//
// A lot of code and many ideas stolen from:
//   miniLA_win
//
//   (c) miniLA Team
//
// =====================================================
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this package; see the file COPYING.  If not, write to
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.




unit mlausb;

{$mode objfpc}{$H+}

interface
uses
  Classes, SysUtils, Dialogs, usb_io, D2XXUnit, uglobdat;

function mla_init:boolean;
function mla_close:boolean;
procedure mla_ReadStatus(bothregs:boolean);
procedure mla_Reset;
procedure mla_SetFreq;
procedure mla_SetRegs;
procedure mla_StartSample;
procedure mla_StoppSample;
procedure mla_ReadData_PrePost;
procedure mla_ReadData_NoPre;
procedure mla_doze;



implementation


function mla_init:boolean;
begin
   mla_init:=false;
   if mla_connected then usb_close;
   try
     usb_init;
   except
     on E: Exception do begin
       commerror:=true;
       MessageDlg(E.Message,mtError,[mbOk],0);
       exit;
     end;
   end;
   mla_init:=true;
   // reset minila:
   Usb_WriteReg(0,$40,true); // reset = 1
   Usb_Writereg(0,0,true);  // 0 again
end;

function mla_close:boolean;
begin usb_close;
      mla_close:=true;
end;

procedure mla_ReadStatus(bothregs:boolean);
begin
   if bothregs then begin  // read minila-version:
      Usb_ReadReg(1,true);
      mlaregs.Version:= (FT_In_Buffer[0] and $7F); end;
   Usb_ReadReg(3,true); // Statusreg2
   mlaregs.Status:=(FT_In_Buffer[0] and %11111000);
end;

procedure mla_Reset;
begin
   // reset minila:
   Usb_WriteReg(0,$40,true); // reset = 1
   Usb_Writereg(0,0,true);  // 0 again
end;

procedure mla_SetFreq;
begin   // fallingEdge still missing!
  Usb_WriteReg(3,(mlaregs.TimeB and $3F),true);
end;

procedure mla_SetRegs;
begin
  Usb_WriteReg(0,mlaregs.CTRL,true);
  Usb_WriteReg(1,(mlaregs.TrigEC and $0F),true);
  Usb_WriteReg(2,(mlaregs.TrigLC and $0F),true);
  Usb_WriteReg(3,(mlaregs.TimeB and $3F),true);
  Usb_WriteReg(4,(mlaregs.PrePost and $1F),true);
  Usb_WriteReg(5,mlaregs.TrigVal1,true);
  Usb_WriteReg(6,mlaregs.TrigVal2,true);
  Usb_WriteReg(7,mlaregs.TrigEdg1,true);
  Usb_WriteReg(8,mlaregs.TrigEdg2,true);
  Usb_WriteReg(9,mlaregs.TrigMask1,true);
  Usb_WriteReg(10,mlaregs.TrigMask2,true);
  Usb_WriteReg(13,mlaregs.TrigCTRL,true);
end;

procedure mla_Startsample;
begin
  mlaregs.CTRL:=$80;
  Usb_WriteReg(0,mlaregs.CTRL,true);
end;

procedure mla_Stoppsample;
begin
  mlaregs.CTRL:=$08;
  Usb_WriteReg(0,mlaregs.CTRL,true);
end;

procedure mla_ReadData_PrePost;
var n,d_i,dn,s,r_i:longint;
    psize:integer;
begin
   datalength:=0;
   mla_ReadStatus(false);
   if (mlaregs.Status and 8)<>0 then begin  // 128 k of data
     Usb_WriteReg(0,16,true); //AutoIncrement
     n:=0; d_i:=0;
     while n < (maxdatal-1) do begin
      dn:=0; psize:=$600;
      if (maxdatal-n-1)<psize then psize:=maxdatal-n-1;
      for s:=0 to (psize-1) do begin
        Usb_ReadReg(0,false);
        Usb_ReadReg(0,false);
        Usb_ReadReg(0,false);
        if s=(psize-1) then
          dn:=Usb_ReadReg(0,true)
        else Usb_ReadReg(0,false);
      end;
      // check number of received data
      if dn<>(psize*4) then begin
        if d_i<>0 then datalength:=d_i
        else datalength:=1;
        exit;
      end;
      // read input buffer
      r_i:=0;
      for s:=0 to (psize-1) do begin
        n:=n+1;
        Chandat1[d_i]:=FT_In_Buffer[r_i];
        Chandat2[d_i]:=FT_In_Buffer[r_i+1];
        Chandat3[d_i]:=FT_In_Buffer[r_i+2];
        Chandat4[d_i]:=FT_In_Buffer[r_i+3];
        d_i:=d_i+1;
        r_i:=r_i+4;
      end;
     end;
     if d_i<>0 then datalength:=d_i
     else datalength:=1;
   end;
   //reset MiniLA
   sleep(ftd_latency);
   mla_reset;
end;

procedure mla_ReadData_NoPre;
var n,j,d_i,dn,s,s_max,r_i:longint; i,psize,p:integer;
begin mla_ReadStatus(false);
   if (mlaregs.Status and 8) = 0 then begin
     j:=maxdatal-1;
     i:=31-mlaregs.PrePost; //chunks of 8 kBytes to skip
     if i >0 then begin //if any
       s:=1; s_max:=(i*$2000-7);
       while s<s_max do begin
         psize:=$600;
         if psize>(s_max-s) then
           psize:=(s_max-s);
         for p:=1 to psize do begin
           if p=psize then
             Usb_WriteReg(0,32,true)
           else Usb_WriteReg(0,32,false);
           Chandat1[j]:=0; Chandat2[j]:=0;
           Chandat3[j]:=0; Chandat4[j]:=0;
           j:=j-1;
         end;
         s:=s+psize;
       end;
     end;
     mla_ReadStatus(false); // SCT=1?
     while ((mlaregs.Status and 8)=0) and (j>0) do begin
         Usb_WriteReg(0,32,true); //increment memory address
         Chandat1[j]:=0; Chandat2[j]:=0;
         Chandat3[j]:=0; Chandat4[j]:=0;
         mla_ReadStatus(false);
         j:=j-1;
     end;
     for i:=1 to 3 do begin //3 more times
         Usb_WriteReg(0,32,true);
         Chandat1[j]:=0; Chandat2[j]:=0;
         Chandat3[j]:=0; Chandat4[j]:=0;
         j:=j-1;
     end;
     Usb_WriteReg(0,16,true); //AutoIncrement
     n:=0; d_i:=0;
     while n < j do begin
      dn:=0; psize:=$600;
      if (j-n)<psize then psize:=j-n;
      for s:=0 to (psize-1) do begin
        Usb_ReadReg(0,false);
        Usb_ReadReg(0,false);
        Usb_ReadReg(0,false);
        if s=(psize-1) then
          dn:=Usb_ReadReg(0,true)
        else Usb_ReadReg(0,false);
      end;
      // check number of received data
      if dn<>(psize*4) then begin
        if d_i<>0 then datalength:=d_i
        else datalength:=1;
        exit;
      end;
      // read input buffer
      r_i:=0;
      for s:=0 to (psize-1) do begin
        n:=n+1;
        Chandat1[d_i]:=FT_In_Buffer[r_i];
        Chandat2[d_i]:=FT_In_Buffer[r_i+1];
        Chandat3[d_i]:=FT_In_Buffer[r_i+2];
        Chandat4[d_i]:=FT_In_Buffer[r_i+3];
        d_i:=d_i+1;
        r_i:=r_i+4;
      end;
     end;
     if d_i<>0 then datalength:=d_i
     else datalength:=1;
   end;
   //reset MiniLA
   sleep(ftd_latency);
   mla_reset;
end;

procedure mla_doze;

begin
    Usb_WriteReg(3,15,true);
end;

end.

